#The script must include the specific command for job submission used in you batch system followed by $1.
#In the case of ts this is just ts plus the appropriate gaussian command followed by $1 

ts bash $1